@@ -30,6 +30,7 @@ require 'excon/response'
 require 'excon/middlewares/decompress'
 require 'excon/middlewares/escape_path'
 require 'excon/middlewares/redirect_follower'
+require 'excon/middlewares/capture_cookies'
 require 'excon/pretty_printer'
 require 'excon/socket'
 require 'excon/ssl_socket'
@@ -211,7 +212,12 @@ module Excon
 
     # get a list of defined stubs
     def stubs
-      @stubs ||= []
+      case Excon.defaults[:stubs]
+      when :global
+        @stubs ||= []
+      when :local
+        Thread.current[:_excon_stubs] ||= []
+      end
     end
 
     # remove first/oldest stub matching request_params
