@@ -140,9 +140,9 @@ module Excon
 
           # add additional "\r\n" to indicate end of headers
           request << CR_NL
-          socket.write(request) # write out request + headers
 
           if datum.has_key?(:request_block)
+            socket.write(request) # write out request + headers
             while true # write out body with chunked encoding
               chunk = datum[:request_block].call
               if FORCE_ENC
@@ -155,13 +155,30 @@ module Excon
                 break
               end
             end
-          elsif !body.nil? # write out body
+          elsif body.nil?
+            socket.write(request) # write out request + headers
+          else # write out body
             if body.respond_to?(:binmode)
               body.binmode
             end
             if body.respond_to?(:rewind)
               body.rewind  rescue nil
             end
+
+            # if request + headers is less than chunk size, fill with body
+            if FORCE_ENC
+              request.force_encoding('BINARY')
+            end
+            chunk = body.read([datum[:chunk_size] - request.length, 0].max)
+            if chunk
+              if FORCE_ENC
+                chunk.force_encoding('BINARY')
+              end
+              socket.write(request << chunk)
+            else
+              socket.write(request) # write out request + headers
+            end
+
             while chunk = body.read(datum[:chunk_size])
               socket.write(chunk)
             end
@@ -172,7 +189,7 @@ module Excon
         when Excon::Errors::StubNotFound, Excon::Errors::Timeout
           raise(error)
         else
-          raise(Excon::Errors::SocketError.new(error))
+          raise_socket_error(error)
         end
       end
 
@@ -207,7 +224,7 @@ module Excon
       datum[:headers] = @data[:headers].merge(datum[:headers] || {})
 
       if datum[:scheme] == UNIX
-        datum[:headers]['Host']   ||= '' << datum[:socket]
+        datum[:headers]['Host']   = ''
       else
         datum[:headers]['Host']   ||= '' << datum[:host] << port_string(datum)
       end
@@ -375,7 +392,7 @@ module Excon
       when Excon::Errors::HTTPStatusError, Excon::Errors::Timeout
         raise(error)
       else
-        raise(Excon::Errors::SocketError.new(error))
+        raise_socket_error(error)
       end
     end
 
@@ -383,7 +400,7 @@ module Excon
       unix_proxy = @data[:proxy] ? @data[:proxy][:scheme] == UNIX : false
       sockets[@socket_key] ||= if @data[:scheme] == UNIX || unix_proxy
         Excon::UnixSocket.new(@data)
-      elsif @data[:scheme] == HTTPS
+      elsif @data[:ssl_uri_schemes].include?(@data[:scheme])
         Excon::SSLSocket.new(@data)
       else
         Excon::Socket.new(@data)
@@ -398,6 +415,14 @@ module Excon
       end
     end
 
+    def raise_socket_error(error)
+      if error.message =~ /certificate verify failed/
+        raise(Excon::Errors::CertificateError.new(error))
+      else
+        raise(Excon::Errors::SocketError.new(error))
+      end
+    end
+
     def setup_proxy
       if @data[:disable_proxy]
         if @data[:proxy]
@@ -422,6 +447,8 @@ module Excon
         case @data[:proxy]
         when nil
           @data.delete(:proxy)
+        when ''
+          @data.delete(:proxy)
         when Hash
           # no processing needed
         when String, URI
