@@ -1,6 +1,6 @@
 module Excon
 
-  VERSION = '0.45.1'
+  VERSION = '0.49.0'
 
   CR_NL = "\r\n"
 
@@ -132,6 +132,8 @@ module Excon
     :read_timeout         => 60,
     :retry_limit          => DEFAULT_RETRY_LIMIT,
     :ssl_verify_peer      => true,
+    :ssl_uri_schemes      => [HTTPS],
+    :stubs                => :global,
     :tcp_nodelay          => false,
     :thread_safe_sockets  => true,
     :uri_parser           => URI,
