@@ -3,16 +3,34 @@ module Excon
 
     class Error < StandardError; end
     class StubNotFound < StandardError; end
+    class InvalidStub < StandardError; end
 
     class SocketError < Error
       attr_reader :socket_error
 
-      def initialize(socket_error=nil)
-        if socket_error.message =~ /certificate verify failed/
-          super("Unable to verify certificate, please set `Excon.defaults[:ssl_ca_path] = path_to_certs`, `ENV['SSL_CERT_DIR'] = path_to_certs`, `Excon.defaults[:ssl_ca_file] = path_to_file`, `ENV['SSL_CERT_FILE'] = path_to_file` or `Excon.defaults[:ssl_verify_peer] = false` (less secure).")
+      def initialize(socket_error = Excon::Errors::Error.new)
+        if is_a?(CertificateError)
+          super
         else
           super("#{socket_error.message} (#{socket_error.class})")
+          set_backtrace(socket_error.backtrace)
+          @socket_error = socket_error
         end
+      end
+    end
+
+    class CertificateError < SocketError
+      def initialize(socket_error = Excon::Errors::Error.new)
+        msg = "Unable to verify certificate. This may be an issue with the remote host or with Excon." +
+              "Excon has certificates bundled, but these can be customized." +
+              "`Excon.defaults[:ssl_ca_path] = path_to_certs`, " +
+              "`ENV['SSL_CERT_DIR'] = path_to_certs`, " +
+              "`Excon.defaults[:ssl_ca_file] = path_to_file`, " +
+              "`ENV['SSL_CERT_FILE'] = path_to_file`, " +
+              "`Excon.defaults[:ssl_verify_callback] = callback` (see OpenSSL::SSL::SSLContext#verify_callback), or " +
+              "`Excon.defaults[:ssl_verify_peer] = false` (less secure)."
+        full_message = "#{socket_error.message} (#{socket_error.class})" + ' ' + msg
+        super(full_message)
         set_backtrace(socket_error.backtrace)
         @socket_error = socket_error
       end
@@ -35,7 +53,7 @@ module Excon
         @response = response
       end
     end
-    
+
     # HTTP Error classes
     class Informational < HTTPStatusError; end
     class Success < HTTPStatusError; end
@@ -78,6 +96,7 @@ module Excon
     class RequestedRangeNotSatisfiable < ClientError; end     # 416
     class ExpectationFailed < ClientError; end                # 417
     class UnprocessableEntity < ClientError; end              # 422
+    class TooManyRequests < ClientError; end                  # 429
     class InternalServerError < ServerError; end              # 500
     class NotImplemented < ServerError; end                   # 501
     class BadGateway < ServerError; end                       # 502
@@ -122,6 +141,7 @@ module Excon
         416 => [Excon::Errors::RequestedRangeNotSatisfiable, 'Request Range Not Satisfiable'],
         417 => [Excon::Errors::ExpectationFailed, 'Expectation Failed'],
         422 => [Excon::Errors::UnprocessableEntity, 'Unprocessable Entity'],
+        429 => [Excon::Errors::TooManyRequests, 'Too Many Requests'],
         500 => [Excon::Errors::InternalServerError, 'InternalServerError'],
         501 => [Excon::Errors::NotImplemented, 'Not Implemented'],
         502 => [Excon::Errors::BadGateway, 'Bad Gateway'],
@@ -129,24 +149,23 @@ module Excon
         504 => [Excon::Errors::GatewayTimeout, 'Gateway Timeout']
       }
 
-      error, message = @errors[response[:status]] || [Excon::Errors::HTTPStatusError, 'Unknown']
+      error_class, error_message = @errors[response[:status]] || [Excon::Errors::HTTPStatusError, 'Unknown']
 
-      message = "Expected(#{request[:expects].inspect}) <=> Actual(#{response[:status]} #{message})"
+      message = StringIO.new
+      message.puts("Expected(#{request[:expects].inspect}) <=> Actual(#{response[:status]} #{error_message})")
 
       if request[:debug_request]
-        # scrub authorization
-        req = request.dup
-        req.reject! {|key, value| [:connection, :stack].include?(key)}
-        if req.has_key?(:headers) && req[:headers].has_key?('Authorization')
-          req[:headers] = req[:headers].dup
-          req[:headers]['Authorization'] = REDACTED
-        end
-        message << "\n  request => #{req.inspect}"
+        message.puts('excon.error.request')
+        Excon::PrettyPrinter.pp(message, request)
       end
 
-      message << "\n  response => #{response.inspect}" if request[:debug_response]
+      if request[:debug_response]
+        message.puts('excon.error.response')
+        Excon::PrettyPrinter.pp(message, response.data)
+      end
 
-      error.new(message, request, response)
+      message.rewind
+      error_class.new(message.read, request, response)
     end
 
   end
