@@ -16,11 +16,20 @@ module Excon
     def headers
       @data[:headers]
     end
-    def status=(new_status)
-      @data[:status] = new_status
+    def host
+      @data[:host]
     end
-    def status
-      @data[:status]
+    def local_address
+      @data[:local_address]
+    end
+    def local_port
+      @data[:local_port]
+    end
+    def path
+      @data[:path]
+    end
+    def port
+      @data[:port]
     end
     def reason_phrase=(new_reason_phrase)
       @data[:reason_phrase] = new_reason_phrase
@@ -34,11 +43,17 @@ module Excon
     def remote_ip
       @data[:remote_ip]
     end
-    def local_port
-      @data[:local_port]
+    def status=(new_status)
+      @data[:status] = new_status
     end
-    def local_address
-      @data[:local_address]
+    def status
+      @data[:status]
+    end
+    def status_line
+      @data[:status_line]
+    end
+    def status_line=(new_status_line)
+      @data[:status_line] = new_status_line
     end
 
     def self.parse(socket, datum)
@@ -51,8 +66,13 @@ module Excon
 
       datum[:response] = {
         :body          => '',
+        :cookies       => [],
+        :host          => datum[:host],
         :headers       => Excon::Headers.new,
+        :path          => datum[:path],
+        :port          => datum[:port],
         :status        => status,
+        :status_line   => line,
         :reason_phrase => reason_phrase
       }
 
@@ -93,25 +113,27 @@ module Excon
           if response_block
             while (chunk_size = socket.readline.chomp!.to_i(16)) > 0
               while chunk_size > 0
-                chunk = socket.read(chunk_size)
+                chunk = socket.read(chunk_size) || raise(EOFError)
                 chunk_size -= chunk.bytesize
                 response_block.call(chunk, nil, nil)
               end
               new_line_size = 2 # 2 == "\r\n".length
               while new_line_size > 0
-                new_line_size -= socket.read(new_line_size).length
+                chunk = socket.read(new_line_size) || raise(EOFError)
+                new_line_size -= chunk.length
               end
             end
           else
             while (chunk_size = socket.readline.chomp!.to_i(16)) > 0
               while chunk_size > 0
-                chunk = socket.read(chunk_size)
+                chunk = socket.read(chunk_size) || raise(EOFError)
                 chunk_size -= chunk.bytesize
                 datum[:response][:body] << chunk
               end
               new_line_size = 2 # 2 == "\r\n".length
               while new_line_size > 0
-                new_line_size -= socket.read(new_line_size).length
+                chunk = socket.read(new_line_size) || raise(EOFError)
+                new_line_size -= chunk.length
               end
             end
           end
@@ -124,13 +146,13 @@ module Excon
           if remaining = content_length
             if response_block
               while remaining > 0
-                chunk = socket.read([datum[:chunk_size], remaining].min)
+                chunk = socket.read([datum[:chunk_size], remaining].min) || raise(EOFError)
                 response_block.call(chunk, [remaining - chunk.bytesize, 0].max, content_length)
                 remaining -= chunk.bytesize
               end
             else
               while remaining > 0
-                chunk = socket.read([datum[:chunk_size], remaining].min)
+                chunk = socket.read([datum[:chunk_size], remaining].min) || raise(EOFError)
                 datum[:response][:body] << chunk
                 remaining -= chunk.bytesize
               end
@@ -153,7 +175,7 @@ module Excon
 
     def self.parse_headers(socket, datum)
       last_key = nil
-      until (data = socket.readline.chomp!).empty?
+      until (data = socket.readline.chomp).empty?
         if !data.lstrip!.nil?
           raise Excon::Errors::ResponseParseError, 'malformed header' unless last_key
           # append to last_key's last value
@@ -163,6 +185,9 @@ module Excon
           raise Excon::Errors::ResponseParseError, 'malformed header' unless value
           # add key/value or append value to existing values
           datum[:response][:headers][key] = ([datum[:response][:headers][key]] << value.strip).compact.join(', ')
+          if key.casecmp('Set-Cookie') == 0
+            datum[:response][:cookies] << value.strip
+          end
           last_key = key
         end
       end
