@@ -252,5 +252,42 @@ Shindo.tests('Excon stubs') do
     Excon.stubs.clear
   end
 
+  tests("global stubs") do
+    connection = Excon.new('http://127.0.0.1:9292', :mock => true)
+    Excon.stub({}, {:body => '1'})
+    t = Thread.new do
+      Excon.stub({}, {:body => '2'})
+      connection.request(:method => :get).body
+    end
+    tests("get on a different thread").returns('2') do
+      t.join.value
+    end
+    tests("get on main thread").returns('2') do
+      connection.request(:method => :get).body
+    end
+    Excon.stubs.clear
+  end
+
+  tests("thread-local stubs") do
+    original_stubs_value = Excon.defaults[:stubs]
+    Excon.defaults[:stubs] = :local
+
+    connection = Excon.new('http://127.0.0.1:9292', :mock => true)
+    Excon.stub({}, {:body => '1'})
+    t = Thread.new do
+      Excon.stub({}, {:body => '2'})
+      connection.request(:method => :get).body
+    end
+    tests("get on a different thread").returns('2') do
+      t.join.value
+    end
+    tests("get on main thread").returns('1') do
+      connection.request(:method => :get).body
+    end
+    Excon.stubs.clear
+
+    Excon.defaults[:stubs] = original_stubs_value
+  end
+
   env_restore
 end
